@@ -243,7 +243,9 @@ load_image (task_t t,
     }
 
   read (fd, &hdr, sizeof hdr);
-  if (*(Elf32_Word *) hdr.e.e_ident == *(Elf32_Word *) "\177ELF")
+  /* File must have magic ELF number.  */
+  if (hdr.e.e_ident[0] == 0177 && hdr.e.e_ident[1] == 'E' &&
+      hdr.e.e_ident[2] == 'L' && hdr.e.e_ident[3] == 'F')
     {
       Elf32_Phdr phdrs[hdr.e.e_phnum], *ph;
       lseek (fd, hdr.e.e_phoff, SEEK_SET);
@@ -1599,13 +1601,17 @@ S_io_reauthenticate (mach_port_t object,
 				MACH_MSG_TYPE_MAKE_SEND);
   assert_perror (err);
 
-  if (! auth_server_authenticate (authserver,
+  do
+    err = auth_server_authenticate (authserver,
 				  rend, MACH_MSG_TYPE_COPY_SEND,
 				  object, MACH_MSG_TYPE_COPY_SEND,
 				  &gu, &gulen,
 				  &au, &aulen,
 				  &gg, &gglen,
-				  &ag, &aglen))
+				  &ag, &aglen);
+  while (err == EINTR);
+
+  if (!err)
     {
       mig_deallocate ((vm_address_t) gu, gulen * sizeof *gu);
       mig_deallocate ((vm_address_t) au, aulen * sizeof *gu);
