@@ -48,6 +48,9 @@
 #include "input.h"
 
 #include "fs_notify_U.h"
+#include "libnetfs/fs_S.h"
+#include "libnetfs/io_S.h"
+#include "tioctl_S.h"
 
 const char *argp_program_version = STANDARD_HURD_VERSION (console);
 
@@ -412,48 +415,52 @@ new_node (struct node **np, vcons_t vcons, vcons_node_type type)
 
 /* Node management.  */
 
+/* We need to drop the soft references on NP.  */
+void
+netfs_try_dropping_softrefs (struct node *np)
+{
+  vcons_t vcons = np->nn->vcons;
+  int release = FALSE;
+
+  pthread_mutex_lock (&vcons->lock);
+  if (np == vcons->dir_node)
+    {
+      release = TRUE;
+      vcons->dir_node = 0;
+    }
+  else if (np == vcons->cons_node)
+    {
+      release = TRUE;
+      vcons->cons_node = 0;
+    }
+  else if (np == vcons->disp_node)
+    {
+      release = TRUE;
+      vcons->disp_node = 0;
+    }
+  else if (np == vcons->inpt_node)
+    {
+      release = TRUE;
+      vcons->inpt_node = 0;
+    }
+  if (release)
+    netfs_nrele_light (np);
+  pthread_mutex_unlock (&vcons->lock);
+
+  /* Release our reference.  */
+  if (release)
+    vcons_release (vcons);
+
+}
+
 /* Node NP has no more references; free all its associated
    storage.  */
 void
 netfs_node_norefs (struct node *np)
 {
-  vcons_t vcons = np->nn->vcons;
-
   /* The root node does never go away.  */
   assert (!np->nn->cons && np->nn->vcons);
 
-  /* Avoid deadlock.  */
-  pthread_spin_unlock (&netfs_node_refcnt_lock);
-
-  /* Find the back reference to ourself in the virtual console
-     structure, and delete it.  */
-  pthread_mutex_lock (&vcons->lock);
-  pthread_spin_lock (&netfs_node_refcnt_lock);
-  if (np->references)
-    {
-      /* Someone else got a reference while we were attempting to go
-	 away.  This can happen in netfs_attempt_lookup.  In this
-	 case, just unlock the node and do nothing else.  */
-      pthread_mutex_unlock (&vcons->lock);
-      pthread_mutex_unlock (&np->lock);
-      return;
-    }
-  if (np == vcons->dir_node)
-    vcons->dir_node = 0;
-  else if (np == vcons->cons_node)
-    vcons->cons_node = 0;
-  else if (np == vcons->disp_node)
-    vcons->disp_node = 0;
-  else
-    {
-      assert (np == vcons->inpt_node);
-      vcons->inpt_node = 0;
-    }
-  pthread_mutex_unlock (&vcons->lock);
-
-  /* Release our reference.  */
-  vcons_release (vcons);
-
   free (np->nn);
   free (np);
 }
@@ -631,7 +638,10 @@ netfs_attempt_lookup (struct iouser *user, struct node *dir,
 	     the virtual console.  */
 	  err = new_node (node, vcons, VCONS_NODE_DIR);
 	  if (!err)
-	    vcons->dir_node = *node;
+            {
+              vcons->dir_node = *node;
+              netfs_nref_light (*node);
+            }
 	  else
 	    release_vcons = 1;
 	}
@@ -660,6 +670,7 @@ netfs_attempt_lookup (struct iouser *user, struct node *dir,
 	      if (!err)
 		{
 		  vcons->cons_node = *node;
+                  netfs_nref_light (*node);
 		  ref_vcons = 1;
 		}
 	    }
@@ -679,6 +690,7 @@ netfs_attempt_lookup (struct iouser *user, struct node *dir,
 	      if (!err)
 		{
 		  vcons->disp_node = *node;
+                  netfs_nref_light (*node);
 		  ref_vcons = 1;
 		}
 	    }
@@ -698,6 +710,7 @@ netfs_attempt_lookup (struct iouser *user, struct node *dir,
 	      if (!err)
 		{
 		  vcons->inpt_node = *node;
+                  netfs_nref_light (*node);
 		  ref_vcons = 1;
 		}
 	    }
@@ -1815,98 +1828,101 @@ S_tioctl_tiocgpgrp (struct protid *cred, int *pgrp)
 }
 
 kern_return_t
-S_tioctl_tiocmodg (io_t port, int *state)
+S_tioctl_tiocmodg (struct protid *cred, int *state)
 {
   return EOPNOTSUPP;
 }
 
 kern_return_t
-S_tioctl_tiocmods (io_t port, int state)
+S_tioctl_tiocmods (struct protid *cred, int state)
 {
   return EOPNOTSUPP;
 }
 
 kern_return_t
-S_tioctl_tiocexcl (io_t port)
+S_tioctl_tiocexcl (struct protid *cred)
 {
   return EOPNOTSUPP;
 }
 
 kern_return_t
-S_tioctl_tiocnxcl (io_t port)
+S_tioctl_tiocnxcl (struct protid *cred)
 {
   return EOPNOTSUPP;
 }
 
 kern_return_t
-S_tioctl_tiocgeta (io_t port, tcflag_t *modes, cc_t *ccs, speed_t *speeds)
+S_tioctl_tiocgeta (struct protid *cred, tcflag_t *modes, cc_t *ccs,
+		   speed_t *speeds)
 {
   return EOPNOTSUPP;
 }
 
 kern_return_t
-S_tioctl_tiocseta (io_t port, tcflag_t *modes, cc_t *ccs, speed_t *speeds)
+S_tioctl_tiocseta (struct protid *cred, tcflag_t *modes, cc_t *ccs,
+		   speed_t *speeds)
 {
   return EOPNOTSUPP;
 }
 
 kern_return_t
-S_tioctl_tiocsetaw (io_t port, tcflag_t *modes, cc_t *ccs, speed_t *speeds)
+S_tioctl_tiocsetaw (struct protid *cred, tcflag_t *modes, cc_t *ccs,
+		    speed_t *speeds)
 {
   return EOPNOTSUPP;
 }
 
 kern_return_t
-S_tioctl_tiocsetaf (io_t port, tcflag_t *modes, cc_t *ccs,
-				  speed_t *speeds)
+S_tioctl_tiocsetaf (struct protid *cred, tcflag_t *modes, cc_t *ccs,
+		    speed_t *speeds)
 {
   return EOPNOTSUPP;
 }
 
 kern_return_t
-S_tioctl_tiocgetd (io_t port, int *disc)
+S_tioctl_tiocgetd (struct protid *cred, int *disc)
 {
   return EOPNOTSUPP;
 }
 
 kern_return_t
-S_tioctl_tiocsetd (io_t port, int disc)
+S_tioctl_tiocsetd (struct protid *cred, int disc)
 {
   return EOPNOTSUPP;
 }
 
 kern_return_t
-S_tioctl_tiocdrain (io_t port)
+S_tioctl_tiocdrain (struct protid *cred)
 {
   return EOPNOTSUPP;
 }
 
 kern_return_t
-S_tioctl_tiocmget (io_t port, int *bits)
+S_tioctl_tiocmget (struct protid *cred, int *bits)
 {
   return EOPNOTSUPP;
 }
 
 kern_return_t
-S_tioctl_tiocmset (io_t port, int bits)
+S_tioctl_tiocmset (struct protid *cred, int bits)
 {
   return EOPNOTSUPP;
 }
 
 kern_return_t
-S_tioctl_tiocsig (io_t port, int sig)
+S_tioctl_tiocsig (struct protid *cred, int sig)
 {
   return EOPNOTSUPP;
 }
 
 kern_return_t
-S_tioctl_tiocext (io_t port, int mode)
+S_tioctl_tiocext (struct protid *cred, int mode)
 {
   return EOPNOTSUPP;
 }
 
 kern_return_t
-S_tioctl_tiocucntl (io_t port, int mode)
+S_tioctl_tiocucntl (struct protid *cred, int mode)
 
 {
   return EOPNOTSUPP;
