@@ -99,7 +99,7 @@ diskfs_null_dirstat (struct dirstat *ds)
 
 static error_t
 dirscanblock (vm_address_t blockoff, struct node *dp, int idx,
-	      const char *name, int namelen, enum lookup_type type,
+	      const char *name, size_t namelen, enum lookup_type type,
 	      struct dirstat *ds, ino_t *inum);
 
 
@@ -137,16 +137,16 @@ diskfs_lookup_hard (struct node *dp, const char *name, enum lookup_type type,
 {
   error_t err;
   ino_t inum;
-  int namelen;
+  size_t namelen;
   int spec_dotdot;
   struct node *np = 0;
-  int retry_dotdot = 0;
+  ino_t retry_dotdot = 0;
   vm_prot_t prot =
     (type == LOOKUP) ? VM_PROT_READ : (VM_PROT_READ | VM_PROT_WRITE);
   memory_object_t memobj;
   vm_address_t buf = 0;
   vm_size_t buflen = 0;
-  int blockaddr;
+  vm_address_t blockaddr;
   int idx, lastidx;
   int looped;
 
@@ -195,13 +195,15 @@ diskfs_lookup_hard (struct node *dp, const char *name, enum lookup_type type,
   err = vm_map (mach_task_self (),
 		&buf, buflen, 0, 1, memobj, 0, 0, prot, prot, 0);
   mach_port_deallocate (mach_task_self (), memobj);
+  if (err)
+    return err;
 
   inum = 0;
 
   diskfs_set_node_atime (dp);
 
-  /* Start the lookup at DP->dn->dir_idx.  */
-  idx = dp->dn->dir_idx;
+  /* Start the lookup at diskfs_node_disknode (DP)->dir_idx.  */
+  idx = diskfs_node_disknode (dp)->dir_idx;
   if (idx * DIRBLKSIZ > dp->dn_stat.st_size)
     idx = 0;			/* just in case */
   blockaddr = buf + idx * DIRBLKSIZ;
@@ -215,7 +217,7 @@ diskfs_lookup_hard (struct node *dp, const char *name, enum lookup_type type,
       err = dirscanblock (blockaddr, dp, idx, name, namelen, type, ds, &inum);
       if (!err)
 	{
-	  dp->dn->dir_idx = idx;
+	  diskfs_node_disknode (dp)->dir_idx = idx;
 	  break;
 	}
       if (err != ENOENT)
@@ -304,7 +306,7 @@ diskfs_lookup_hard (struct node *dp, const char *name, enum lookup_type type,
 
       /* Here below are the spec dotdot cases. */
       else if (type == RENAME || type == REMOVE)
-	np = ifind (inum);
+	np = diskfs_cached_ifind (inum);
 
       else if (type == LOOKUP)
 	{
@@ -357,7 +359,7 @@ diskfs_lookup_hard (struct node *dp, const char *name, enum lookup_type type,
 		diskfs_nput (np);
 	    }
 	  else if (type == RENAME || type == REMOVE)
-	    /* We just did ifind to get np; that allocates
+	    /* We just did diskfs_cached_ifind to get np; that allocates
 	       no new references, so we don't have anything to do */
 	    ;
 	  else if (type == LOOKUP)
@@ -377,11 +379,11 @@ diskfs_lookup_hard (struct node *dp, const char *name, enum lookup_type type,
    return ENOENT.  */
 static error_t
 dirscanblock (vm_address_t blockaddr, struct node *dp, int idx,
-	      const char *name, int namelen, enum lookup_type type,
+	      const char *name, size_t namelen, enum lookup_type type,
 	      struct dirstat *ds, ino_t *inum)
 {
-  int nfree = 0;
-  int needed = 0;
+  size_t nfree = 0;
+  size_t needed = 0;
   vm_address_t currentoff, prevoff;
   struct ext2_dir_entry_2 *entry = 0;
   int nentries = 0;
@@ -411,7 +413,7 @@ dirscanblock (vm_address_t blockaddr, struct node *dp, int idx,
 	  || EXT2_DIR_REC_LEN (entry->name_len) > entry->rec_len
 	  || memchr (entry->name, '\0', entry->name_len))
 	{
-	  ext2_warning ("bad directory entry: inode: %Ld offset: %zd",
+	  ext2_warning ("bad directory entry: inode: %Ld offset: %lu",
 			dp->cache_id,
 			currentoff - blockaddr + idx * DIRBLKSIZ);
 	  return ENOENT;
@@ -419,7 +421,7 @@ dirscanblock (vm_address_t blockaddr, struct node *dp, int idx,
 
       if (looking || countcopies)
 	{
-	  int thisfree;
+	  size_t thisfree;
 
 	  /* Count how much free space this entry has in it. */
 	  if (entry->inode == 0)
@@ -482,17 +484,17 @@ dirscanblock (vm_address_t blockaddr, struct node *dp, int idx,
 
       /* Because we scanned the entire block, we should write
 	 down how many entries there were. */
-      if (!dp->dn->dirents)
+      if (!diskfs_node_disknode (dp)->dirents)
 	{
-	  dp->dn->dirents = malloc ((dp->dn_stat.st_size / DIRBLKSIZ)
-				    * sizeof (int));
+	  diskfs_node_disknode (dp)->dirents =
+	    malloc ((dp->dn_stat.st_size / DIRBLKSIZ) * sizeof (int));
 	  for (i = 0; i < dp->dn_stat.st_size/DIRBLKSIZ; i++)
-	    dp->dn->dirents[i] = -1;
+	    diskfs_node_disknode (dp)->dirents[i] = -1;
 	}
       /* Make sure the count is correct if there is one now. */
-      assert (dp->dn->dirents[idx] == -1
-	      || dp->dn->dirents[idx] == nentries);
-      dp->dn->dirents[idx] = nentries;
+      assert (diskfs_node_disknode (dp)->dirents[idx] == -1
+	      || diskfs_node_disknode (dp)->dirents[idx] == nentries);
+      diskfs_node_disknode (dp)->dirents[idx] = nentries;
 
       return ENOENT;
     }
@@ -525,11 +527,11 @@ diskfs_direnter_hard (struct node *dp, const char *name, struct node *np,
 		      struct dirstat *ds, struct protid *cred)
 {
   struct ext2_dir_entry_2 *new;
-  int namelen = strlen (name);
-  int needed = EXT2_DIR_REC_LEN (namelen);
-  int oldneeded;
+  size_t namelen = strlen (name);
+  size_t needed = EXT2_DIR_REC_LEN (namelen);
+  size_t oldneeded;
   vm_address_t fromoff, tooff;
-  int totfreed;
+  size_t totfreed;
   error_t err;
   size_t oldsize = 0;
 
@@ -575,7 +577,7 @@ diskfs_direnter_hard (struct node *dp, const char *name, struct node *np,
 	{
 	  struct ext2_dir_entry_2 *from = (struct ext2_dir_entry_2 *)fromoff;
 	  struct ext2_dir_entry_2 *to = (struct ext2_dir_entry_2 *) tooff;
-	  int fromreclen = from->rec_len;
+	  size_t fromreclen = from->rec_len;
 
 	  if (from->inode != 0)
 	    {
@@ -651,7 +653,7 @@ diskfs_direnter_hard (struct node *dp, const char *name, struct node *np,
   memcpy (new->name, name, namelen);
 
   /* Mark the directory inode has having been written.  */
-  dp->dn->info.i_flags &= ~EXT2_BTREE_FL;
+  diskfs_node_disknode (dp)->info.i_flags &= ~EXT2_BTREE_FL;
   dp->dn_set_mtime = 1;
 
   munmap ((caddr_t) ds->mapbuf, ds->mapextent);
@@ -660,33 +662,34 @@ diskfs_direnter_hard (struct node *dp, const char *name, struct node *np,
     {
       /* If we are keeping count of this block, then keep the count up
 	 to date. */
-      if (dp->dn->dirents && dp->dn->dirents[ds->idx] != -1)
-	dp->dn->dirents[ds->idx]++;
+      if (diskfs_node_disknode (dp)->dirents
+	  && diskfs_node_disknode (dp)->dirents[ds->idx] != -1)
+	diskfs_node_disknode (dp)->dirents[ds->idx]++;
     }
   else
     {
       int i;
       /* It's cheap, so start a count here even if we aren't counting
 	 anything at all. */
-      if (dp->dn->dirents)
+      if (diskfs_node_disknode (dp)->dirents)
 	{
-	  dp->dn->dirents = realloc (dp->dn->dirents,
-				     (dp->dn_stat.st_size / DIRBLKSIZ
-				      * sizeof (int)));
+	  diskfs_node_disknode (dp)->dirents =
+	    realloc (diskfs_node_disknode (dp)->dirents,
+		     (dp->dn_stat.st_size / DIRBLKSIZ * sizeof (int)));
 	  for (i = oldsize / DIRBLKSIZ;
 	       i < dp->dn_stat.st_size / DIRBLKSIZ;
 	       i++)
-	    dp->dn->dirents[i] = -1;
+	    diskfs_node_disknode (dp)->dirents[i] = -1;
 
-	  dp->dn->dirents[ds->idx] = 1;
+	  diskfs_node_disknode (dp)->dirents[ds->idx] = 1;
 	}
       else
 	{
-	  dp->dn->dirents = malloc (dp->dn_stat.st_size / DIRBLKSIZ
-				    * sizeof (int));
+	  diskfs_node_disknode (dp)->dirents =
+	    malloc (dp->dn_stat.st_size / DIRBLKSIZ * sizeof (int));
 	  for (i = 0; i < dp->dn_stat.st_size / DIRBLKSIZ; i++)
-	    dp->dn->dirents[i] = -1;
-	  dp->dn->dirents[ds->idx] = 1;
+	    diskfs_node_disknode (dp)->dirents[i] = -1;
+	  diskfs_node_disknode (dp)->dirents[ds->idx] = 1;
 	}
     }
 
@@ -718,14 +721,15 @@ diskfs_dirremove_hard (struct node *dp, struct dirstat *ds)
     }
 
   dp->dn_set_mtime = 1;
-  dp->dn->info.i_flags &= ~EXT2_BTREE_FL;
+  diskfs_node_disknode (dp)->info.i_flags &= ~EXT2_BTREE_FL;
 
   munmap ((caddr_t) ds->mapbuf, ds->mapextent);
 
   /* If we are keeping count of this block, then keep the count up
      to date. */
-  if (dp->dn->dirents && dp->dn->dirents[ds->idx] != -1)
-    dp->dn->dirents[ds->idx]--;
+  if (diskfs_node_disknode (dp)->dirents
+      && diskfs_node_disknode (dp)->dirents[ds->idx] != -1)
+    diskfs_node_disknode (dp)->dirents[ds->idx]--;
 
   diskfs_file_update (dp, diskfs_synchronous);
 
@@ -749,7 +753,7 @@ diskfs_dirrewrite_hard (struct node *dp, struct node *np, struct dirstat *ds)
 
   ds->entry->inode = np->cache_id;
   dp->dn_set_mtime = 1;
-  dp->dn->info.i_flags &= ~EXT2_BTREE_FL;
+  diskfs_node_disknode (dp)->info.i_flags &= ~EXT2_BTREE_FL;
 
   munmap ((caddr_t) ds->mapbuf, ds->mapextent);
 
@@ -821,7 +825,7 @@ diskfs_drop_dirstat (struct node *dp, struct dirstat *ds)
    write the answer down in its dirents array.  As a side affect
    fill BUF with the block.  */
 static error_t
-count_dirents (struct node *dp, int nb, char *buf)
+count_dirents (struct node *dp, block_t nb, char *buf)
 {
   size_t amt;
   char *offinblk;
@@ -829,7 +833,7 @@ count_dirents (struct node *dp, int nb, char *buf)
   int count = 0;
   error_t err;
 
-  assert (dp->dn->dirents);
+  assert (diskfs_node_disknode (dp)->dirents);
   assert ((nb + 1) * DIRBLKSIZ <= dp->dn_stat.st_size);
 
   err = diskfs_node_rdwr (dp, buf, nb * DIRBLKSIZ, DIRBLKSIZ, 0, 0, &amt);
@@ -846,8 +850,9 @@ count_dirents (struct node *dp, int nb, char *buf)
 	count++;
     }
 
-  assert (dp->dn->dirents[nb] == -1 || dp->dn->dirents[nb] == count);
-  dp->dn->dirents[nb] = count;
+  assert (diskfs_node_disknode (dp)->dirents[nb] == -1
+	  || diskfs_node_disknode (dp)->dirents[nb] == count);
+  diskfs_node_disknode (dp)->dirents[nb] = count;
   return 0;
 }
 
@@ -866,8 +871,8 @@ diskfs_get_directs (struct node *dp,
 		    vm_size_t bufsiz,
 		    int *amt)
 {
-  int blkno;
-  int nblks;
+  block_t blkno;
+  block_t nblks;
   int curentry;
   char buf[DIRBLKSIZ];
   char *bufp;
@@ -882,11 +887,11 @@ diskfs_get_directs (struct node *dp,
 
   nblks = dp->dn_stat.st_size/DIRBLKSIZ;
 
-  if (!dp->dn->dirents)
+  if (!diskfs_node_disknode (dp)->dirents)
     {
-      dp->dn->dirents = malloc (nblks * sizeof (int));
+      diskfs_node_disknode (dp)->dirents = malloc (nblks * sizeof (int));
       for (i = 0; i < nblks; i++)
-	dp->dn->dirents[i] = -1;
+	diskfs_node_disknode (dp)->dirents[i] = -1;
     }
 
   /* Scan through the entries to find ENTRY.  If we encounter
@@ -896,7 +901,7 @@ diskfs_get_directs (struct node *dp,
   bufvalid = 0;
   for (blkno = 0; blkno < nblks; blkno++)
     {
-      if (dp->dn->dirents[blkno] == -1)
+      if (diskfs_node_disknode (dp)->dirents[blkno] == -1)
 	{
 	  err = count_dirents (dp, blkno, buf);
 	  if (err)
@@ -904,11 +909,11 @@ diskfs_get_directs (struct node *dp,
 	  bufvalid = 1;
 	}
 
-      if (curentry + dp->dn->dirents[blkno] > entry)
+      if (curentry + diskfs_node_disknode (dp)->dirents[blkno] > entry)
 	/* ENTRY starts in this block. */
 	break;
 
-      curentry += dp->dn->dirents[blkno];
+      curentry += diskfs_node_disknode (dp)->dirents[blkno];
 
       bufvalid = 0;
     }
