@@ -44,20 +44,23 @@ int diskfs_maxsymlinks = 8;
 static void
 fetch_root ()
 {
-  struct rrip_lookup rl;
-  struct dirrect *dr;
+  struct lookup_context ctx;
+  ino_t id;
   error_t err;
 
-  dr = (struct dirrect *) sblock->root;
+  ctx.dr = (struct dirrect *) sblock->root;
 
   /* First check for SUSP and all relevant extensions */
-  rrip_initialize (dr);
+  rrip_initialize (ctx.dr);
 
   /* Now rescan the node for real */
-  rrip_lookup (dr, &rl, 1);
+  rrip_lookup (ctx.dr, &ctx.rr, 1);
+
+  err = cache_id (ctx.dr, &ctx.rr, &id);
+  assert_perror (err);
 
   /* And fetch the node. */
-  err = load_inode (&diskfs_root_node, dr, &rl);
+  err = diskfs_cached_lookup_context (id, &diskfs_root_node, &ctx);
   assert_perror (err);
 
   pthread_mutex_unlock (&diskfs_root_node->lock);
@@ -69,17 +72,13 @@ static void
 read_sblock ()
 {
   struct voldesc *vd;
-  error_t err;
   struct sblock * volatile sb = 0;
 
-  err = diskfs_catch_exception ();
-  if (err)
-    error (4, err, "reading superblock");
-
   /* Start at logical sector 16 and keep going until
      we find a matching superblock */
   for (vd = disk_image + (logical_sector_size * 16);
-       (void *) vd < disk_image + (logical_sector_size * 500); /* for sanity */
+       (void *) vd < disk_image + (logical_sector_size * 500) /* for sanity */
+         && (void *) vd + logical_sector_size < disk_image + disk_image_len;
        vd = (void *) vd + logical_sector_size)
     {
       if (vd->type == VOLDESC_END)
@@ -102,7 +101,6 @@ read_sblock ()
   if (!sblock)
     error (1, errno, "Could not allocate memory for superblock");
   memcpy (sblock, sb, sizeof (struct sblock));
-  diskfs_end_catch_exception ();
 
   /* Parse some important bits of this */
   logical_block_size = isonum_723 (sblock->blksize);
