@@ -49,7 +49,7 @@ netfs_S_dir_lookup (struct protid *diruser,
   char *nextname;
   char *relpath;
   error_t error;
-  struct protid *newpi;
+  struct protid *newpi = NULL;
   struct iouser *user;
 
   if (!diruser)
@@ -128,7 +128,9 @@ netfs_S_dir_lookup (struct protid *diruser,
 	    *do_retry = FS_RETRY_REAUTH;
 	    *retry_port = diruser->po->shadow_root_parent;
 	    *retry_port_type = MACH_MSG_TYPE_COPY_SEND;
-	    if (! lastcomp)
+	    if (lastcomp && mustbedir) /* Trailing slash.  */
+	      strcpy (retry_name, "/");
+	    else if (!lastcomp)
 	      strcpy (retry_name, nextname);
 	    error = 0;
 	    pthread_mutex_unlock (&dnp->lock);
@@ -142,7 +144,9 @@ netfs_S_dir_lookup (struct protid *diruser,
 	    *do_retry = FS_RETRY_REAUTH;
 	    *retry_port = diruser->po->root_parent;
 	    *retry_port_type = MACH_MSG_TYPE_COPY_SEND;
-	    if (!lastcomp)
+	    if (lastcomp && mustbedir) /* Trailing slash.  */
+	      strcpy (retry_name, "/");
+	    else if (!lastcomp)
 	      strcpy (retry_name, nextname);
 	    error = 0;
 	    pthread_mutex_unlock (&dnp->lock);
@@ -194,7 +198,7 @@ netfs_S_dir_lookup (struct protid *diruser,
       if (error)
 	goto out;
 
-      if ((((flags & O_NOTRANS) == 0) || !lastcomp)
+      if ((((flags & O_NOTRANS) == 0) || !lastcomp || mustbedir)
 	  && ((np->nn_translated & S_IPTRANS)
 	      || S_ISFIFO (np->nn_translated)
 	      || S_ISCHR (np->nn_translated)
@@ -288,10 +292,16 @@ netfs_S_dir_lookup (struct protid *diruser,
 	  if (error != ENOENT)
 	    {
 	      *retry_port_type = MACH_MSG_TYPE_MOVE_SEND;
-	      if (!lastcomp && !error)
+	      if (!error)
 		{
-		  strcat (retry_name, "/");
-		  strcat (retry_name, nextname);
+		  char *end = strchr (retry_name, '\0');
+		  if (mustbedir)
+		    *end++ = '/'; /* Trailing slash.  */
+		  else if (!lastcomp) {
+		    if (end != retry_name)
+		      *end++ = '/';
+		    strcpy (end, nextname);
+		  }
 		}
 
 	      if (register_translator)
@@ -357,7 +367,7 @@ netfs_S_dir_lookup (struct protid *diruser,
 	  linklen = np->nn_stat.st_size;
 
 	  nextnamelen = nextname ? strlen (nextname) + 1 : 0;
-	  newnamelen = nextnamelen + linklen + 1;
+	  newnamelen = nextnamelen + linklen + 1 + 1;
 	  linkbuf = alloca (newnamelen);
 
 	  error = netfs_attempt_readlink (diruser->user, np, linkbuf);
@@ -370,7 +380,13 @@ netfs_S_dir_lookup (struct protid *diruser,
 	      memcpy (linkbuf + linklen + 1, nextname,
 		     nextnamelen - 1);
 	    }
-	  linkbuf[nextnamelen + linklen] = '\0';
+	  if (mustbedir)
+	    {
+	      linkbuf[nextnamelen + linklen] = '/';
+	      linkbuf[nextnamelen + linklen + 1] = '\0';
+	    }
+	  else
+	    linkbuf[nextnamelen + linklen] = '\0';
 
 	  if (linkbuf[0] == '/')
 	    {
@@ -382,6 +398,7 @@ netfs_S_dir_lookup (struct protid *diruser,
 	    }
 
 	  filename = linkbuf;
+	  mustbedir = 0;
 	  if (lastcomp)
 	    {
 	      lastcomp = 0;
