@@ -31,10 +31,36 @@ begin_using_protid_port (file_t port)
   return ports_lookup_port (netfs_port_bucket, port, netfs_protid_class);
 }
 
+static inline struct protid * __attribute__ ((unused))
+begin_using_protid_payload (unsigned long payload)
+{
+  return ports_lookup_payload (netfs_port_bucket, payload, netfs_protid_class);
+}
+
 static inline void __attribute__ ((unused))
 end_using_protid_port (struct protid *cred)
 {
   if (cred)
     ports_port_deref (cred);
 }
+
+static inline struct netfs_control * __attribute__ ((unused))
+begin_using_control_port (fsys_t port)
+{
+  return ports_lookup_port (netfs_port_bucket, port, netfs_control_class);
+}
+
+static inline struct netfs_control * __attribute__ ((unused))
+begin_using_control_payload (unsigned long payload)
+{
+  return ports_lookup_payload (netfs_port_bucket, payload, netfs_control_class);
+}
+
+static inline void __attribute__ ((unused))
+end_using_control_port (struct netfs_control *cred)
+{
+  if (cred)
+    ports_port_deref (cred);
+}
+
 #endif
